'''''''''''''''''''''''''''''''''''''''''''
' Found in Parallax Application Note AN008
'''''''''''''''''''''''''''''''''''''''''''

CON
INP_PIN = 8                                                             'Counter input pin for sigma-delta.
FB_PIN = 9                                                              'Counter feedback pin for sigma-delta.
ADC_INTERVAL = 512                                                      'Time interval over which to accumulate counts.

pub sigma_delta(value)                                                  
                                cognew(@adc_cog, @value)

DAT
org 0
adc_cog                         mov mailbox_ptr,par                     ' save the pointer to the STATIC parameter (HUB) memory area
                                                                        ' the PAR register is initialized by the cognew() function and is a pointer to
                                                                        ' the first STATIC int declared in the C code as the mailbox
                                                                        ' mailbox_ptr will be changed as the code executes. You can reload
                                                                        ' the initial pointer from PAR if you ever need it to point to
                                                                        ' the start of the mailbox again

                                
                                mov frqa,#1                             'Initialize frqa to count up by one.
                                movi ctra,#%0_01001_000                 'Set ctra mode to positive w/feedback.
                                movd ctra,#FB_PIN                       'Write fback pin number to dst field.
                                movs ctra,#INP_PIN                      'Write input pin number to src field.
                                mov dira,fb_mask                        'Make the feedback pin an output.


main_loop                       call #adc                               'Get a new acquisition.
                                wrlong acc,mailbox_ptr
                                jmp #main_loop                          'Back for another.
                                
adc                             mov time,cnt                            'Get the current counter value.
                                add time,#16                            'Add a little to get ahead.
                                waitcnt time,interval                   'Sync to clock; add interval to time.
                                neg acc,phsa                            'Negate phsa into result.
                                waitcnt time,#0                         'Wait for interval to pass.
                                add acc,phsa                            'Add phsa into result.

adc_ret                         ret

mailbox_ptr             long    0                                       ' working ptr into the HUB area - reload from PAR if needed
fb_mask                 long 1 << FB_PIN                                'Mask for feedback pin.
result_addr             long 0                                          'Result address gets plugged in here.
interval                long ADC_INTERVAL                               'Acquisition time.
acc                     res 1                                           'General-purpose accumulator.
time                    res 1                                           'Time variable use for waitcnt.